/*******************************************************************************
* File Name: BCLK.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "BCLK.h"


/*******************************************************************************
* Function Name: BCLK_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void BCLK_Write(uint8 value) 
{
    uint8 staticBits = (BCLK_DR & (uint8)(~BCLK_MASK));
    BCLK_DR = staticBits | ((uint8)(value << BCLK_SHIFT) & BCLK_MASK);
}


/*******************************************************************************
* Function Name: BCLK_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  BCLK_DM_STRONG     Strong Drive 
*  BCLK_DM_OD_HI      Open Drain, Drives High 
*  BCLK_DM_OD_LO      Open Drain, Drives Low 
*  BCLK_DM_RES_UP     Resistive Pull Up 
*  BCLK_DM_RES_DWN    Resistive Pull Down 
*  BCLK_DM_RES_UPDWN  Resistive Pull Up/Down 
*  BCLK_DM_DIG_HIZ    High Impedance Digital 
*  BCLK_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void BCLK_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(BCLK_0, mode);
}


/*******************************************************************************
* Function Name: BCLK_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro BCLK_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 BCLK_Read(void) 
{
    return (BCLK_PS & BCLK_MASK) >> BCLK_SHIFT;
}


/*******************************************************************************
* Function Name: BCLK_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 BCLK_ReadDataReg(void) 
{
    return (BCLK_DR & BCLK_MASK) >> BCLK_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(BCLK_INTSTAT) 

    /*******************************************************************************
    * Function Name: BCLK_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 BCLK_ClearInterrupt(void) 
    {
        return (BCLK_INTSTAT & BCLK_MASK) >> BCLK_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
