/*******************************************************************************
* File Name: EMIF_1.h
* Version 1.30
*
* Description:
*  This file provides constants and parameter values for the EMIF component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2011-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_EMIF_EMIF_1_H)
#define CY_EMIF_EMIF_1_H

#include "EMIF_1.h"
#include "cyfitter.h"
#include "CyLib.h"


/***************************************
*   Conditional Compilation Parameters
****************************************/

/* Check to see if required defines such as CY_PSOC5A are available */
/* They are defined starting with cy_boot v3.0 */

#if !defined (CY_PSOC5A)
    #error Component EMIF_v1_30 requires cy_boot v3.0 or later
#endif /* (CY_ PSOC5A) */

#define EMIF_1_DATA_SIZE      (8u)
#define EMIF_1_ADDR_SIZE      (8u)
#define EMIF_1_MEM_SPEED      (30u)
#define EMIF_1_MODE           (0u)

/* EMIF Modes */
#define EMIF_1_CUSTOM (2u)
#define EMIF_1_ASYNCH (0u)
#define EMIF_1_SYNCH  (1u)

extern uint8 EMIF_1_initVar;


/***************************************
*       Type defines
***************************************/

/* Structure to save state before go to sleep */
typedef struct
{
    uint8  enableState;
} EMIF_1_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
****************************************/

void EMIF_1_Init(void) ;
void EMIF_1_Enable(void) ;
void EMIF_1_Start(void) ;
void EMIF_1_Stop(void) ;
void EMIF_1_SaveConfig(void) ;
void EMIF_1_RestoreConfig(void) ;
void EMIF_1_Sleep(void) ;
void EMIF_1_Wakeup(void) ;
void EMIF_1_ExtMemSleep(void) ;
void EMIF_1_ExtMemWakeup(void) ;


/***************************************
*    Initial Parameter Constants
***************************************/


/***************************************
*           API Constants
***************************************/

#define EMIF_1_CLOCK_DIV          (1u)
#define EMIF_1_READ_WTSTATES      (2u)
#define EMIF_1_WRITE_WTSTATES     (1u)


/***************************************
*             Registers
***************************************/

#define EMIF_1_ENABLE_REG         (*(reg8 *)   EMIF_1_EMIF_ES3__CLOCK_EN)
#define EMIF_1_ENABLE_PTR         ( (reg8 *)   EMIF_1_EMIF_ES3__CLOCK_EN)

#define EMIF_1_MEM_TYPE_REG       (*(reg8 *)   EMIF_1_EMIF_ES3__EM_TYPE)
#define EMIF_1_MEM_TYPE_PTR       ( (reg8 *)   EMIF_1_EMIF_ES3__EM_TYPE)

#if (EMIF_1_MODE == EMIF_1_SYNCH)
    #define EMIF_1_MEM_PWR_REG        (*(reg8 *)   EMIF_1_EMIF_ES3__MEM_DWN)
    #define EMIF_1_MEM_PWR_PTR        ( (reg8 *)   EMIF_1_EMIF_ES3__MEM_DWN)
#endif /* End EMIF_1_MODE == EMIF_1_SYNCH */

#define EMIF_1_CLK_DIV_REG        (*(reg8 *)   EMIF_1_EMIF_ES3__MEMCLK_DIV)
#define EMIF_1_CLK_DIV_PTR        ( (reg8 *)   EMIF_1_EMIF_ES3__MEMCLK_DIV)

#define EMIF_1_NO_UDB_REG         (*(reg8 *)   EMIF_1_EMIF_ES3__NO_UDB)
#define EMIF_1_NO_UDB_PTR         ( (reg8 *)   EMIF_1_EMIF_ES3__NO_UDB)

#define EMIF_1_POWER_REG          (*(reg8 *)   EMIF_1_EMIF_ES3__PM_ACT_CFG)
#define EMIF_1_POWER_PTR          ( (reg8 *)   EMIF_1_EMIF_ES3__PM_ACT_CFG)

#define EMIF_1_STBY_REG           (*(reg8 *)   EMIF_1_EMIF_ES3__PM_STBY_CFG)
#define EMIF_1_STBY_PTR           ( (reg8 *)   EMIF_1_EMIF_ES3__PM_STBY_CFG)

#define EMIF_1_RD_WAIT_STATE_REG  (*(reg8 *)   EMIF_1_EMIF_ES3__RP_WAIT_STATES)
#define EMIF_1_RD_WAIT_STATE_PTR  ( (reg8 *)   EMIF_1_EMIF_ES3__RP_WAIT_STATES)
    
#define EMIF_1_WR_WAIT_STATE_REG  (*(reg8 *)   EMIF_1_EMIF_ES3__WP_WAIT_STATES)
#define EMIF_1_WR_WAIT_STATE_PTR  ( (reg8 *)   EMIF_1_EMIF_ES3__WP_WAIT_STATES)


/***************************************
*       Register Constants
***************************************/

#define EMIF_1_DEFAULT_STATE      (0x00u)
#define EMIF_1_ENABLE             (0x01u)
#define EMIF_1_MEM_SYNC           (0x00u)
#define EMIF_1_MEM_ASYNC          (0x01u)
#define EMIF_1_MODE_UDB           (0x00u)
#define EMIF_1_MODE_NOUDB         (0x01u)
#define EMIF_1_MEM_PWR_DOWN       (0x01u)
#define EMIF_1_POWER_ON           (0x40u)


#endif  /* End CY_EMIF_EMIF_1_H */


/* [] END OF FILE */
