/*******************************************************************************
* File Name: EXEC_LED.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "EXEC_LED.h"


/*******************************************************************************
* Function Name: EXEC_LED_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void EXEC_LED_Write(uint8 value) 
{
    uint8 staticBits = (EXEC_LED_DR & (uint8)(~EXEC_LED_MASK));
    EXEC_LED_DR = staticBits | ((uint8)(value << EXEC_LED_SHIFT) & EXEC_LED_MASK);
}


/*******************************************************************************
* Function Name: EXEC_LED_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  EXEC_LED_DM_STRONG     Strong Drive 
*  EXEC_LED_DM_OD_HI      Open Drain, Drives High 
*  EXEC_LED_DM_OD_LO      Open Drain, Drives Low 
*  EXEC_LED_DM_RES_UP     Resistive Pull Up 
*  EXEC_LED_DM_RES_DWN    Resistive Pull Down 
*  EXEC_LED_DM_RES_UPDWN  Resistive Pull Up/Down 
*  EXEC_LED_DM_DIG_HIZ    High Impedance Digital 
*  EXEC_LED_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void EXEC_LED_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(EXEC_LED_0, mode);
}


/*******************************************************************************
* Function Name: EXEC_LED_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro EXEC_LED_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 EXEC_LED_Read(void) 
{
    return (EXEC_LED_PS & EXEC_LED_MASK) >> EXEC_LED_SHIFT;
}


/*******************************************************************************
* Function Name: EXEC_LED_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 EXEC_LED_ReadDataReg(void) 
{
    return (EXEC_LED_DR & EXEC_LED_MASK) >> EXEC_LED_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(EXEC_LED_INTSTAT) 

    /*******************************************************************************
    * Function Name: EXEC_LED_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 EXEC_LED_ClearInterrupt(void) 
    {
        return (EXEC_LED_INTSTAT & EXEC_LED_MASK) >> EXEC_LED_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
