/*******************************************************************************
* File Name: PLAY_STOP.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "PLAY_STOP.h"


/*******************************************************************************
* Function Name: PLAY_STOP_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void PLAY_STOP_Write(uint8 value) 
{
    uint8 staticBits = (PLAY_STOP_DR & (uint8)(~PLAY_STOP_MASK));
    PLAY_STOP_DR = staticBits | ((uint8)(value << PLAY_STOP_SHIFT) & PLAY_STOP_MASK);
}


/*******************************************************************************
* Function Name: PLAY_STOP_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  PLAY_STOP_DM_STRONG     Strong Drive 
*  PLAY_STOP_DM_OD_HI      Open Drain, Drives High 
*  PLAY_STOP_DM_OD_LO      Open Drain, Drives Low 
*  PLAY_STOP_DM_RES_UP     Resistive Pull Up 
*  PLAY_STOP_DM_RES_DWN    Resistive Pull Down 
*  PLAY_STOP_DM_RES_UPDWN  Resistive Pull Up/Down 
*  PLAY_STOP_DM_DIG_HIZ    High Impedance Digital 
*  PLAY_STOP_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void PLAY_STOP_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(PLAY_STOP_0, mode);
}


/*******************************************************************************
* Function Name: PLAY_STOP_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro PLAY_STOP_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 PLAY_STOP_Read(void) 
{
    return (PLAY_STOP_PS & PLAY_STOP_MASK) >> PLAY_STOP_SHIFT;
}


/*******************************************************************************
* Function Name: PLAY_STOP_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 PLAY_STOP_ReadDataReg(void) 
{
    return (PLAY_STOP_DR & PLAY_STOP_MASK) >> PLAY_STOP_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(PLAY_STOP_INTSTAT) 

    /*******************************************************************************
    * Function Name: PLAY_STOP_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 PLAY_STOP_ClearInterrupt(void) 
    {
        return (PLAY_STOP_INTSTAT & PLAY_STOP_MASK) >> PLAY_STOP_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
