/*******************************************************************************
* File Name: wen.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "wen.h"


/*******************************************************************************
* Function Name: wen_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void wen_Write(uint8 value) 
{
    uint8 staticBits = (wen_DR & (uint8)(~wen_MASK));
    wen_DR = staticBits | ((uint8)(value << wen_SHIFT) & wen_MASK);
}


/*******************************************************************************
* Function Name: wen_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  wen_DM_STRONG     Strong Drive 
*  wen_DM_OD_HI      Open Drain, Drives High 
*  wen_DM_OD_LO      Open Drain, Drives Low 
*  wen_DM_RES_UP     Resistive Pull Up 
*  wen_DM_RES_DWN    Resistive Pull Down 
*  wen_DM_RES_UPDWN  Resistive Pull Up/Down 
*  wen_DM_DIG_HIZ    High Impedance Digital 
*  wen_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void wen_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(wen_0, mode);
}


/*******************************************************************************
* Function Name: wen_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro wen_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 wen_Read(void) 
{
    return (wen_PS & wen_MASK) >> wen_SHIFT;
}


/*******************************************************************************
* Function Name: wen_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 wen_ReadDataReg(void) 
{
    return (wen_DR & wen_MASK) >> wen_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(wen_INTSTAT) 

    /*******************************************************************************
    * Function Name: wen_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 wen_ClearInterrupt(void) 
    {
        return (wen_INTSTAT & wen_MASK) >> wen_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
