/*******************************************************************************
* File Name: CF_CD.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "CF_CD.h"


/*******************************************************************************
* Function Name: CF_CD_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void CF_CD_Write(uint8 value) 
{
    uint8 staticBits = (CF_CD_DR & (uint8)(~CF_CD_MASK));
    CF_CD_DR = staticBits | ((uint8)(value << CF_CD_SHIFT) & CF_CD_MASK);
}


/*******************************************************************************
* Function Name: CF_CD_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  CF_CD_DM_STRONG     Strong Drive 
*  CF_CD_DM_OD_HI      Open Drain, Drives High 
*  CF_CD_DM_OD_LO      Open Drain, Drives Low 
*  CF_CD_DM_RES_UP     Resistive Pull Up 
*  CF_CD_DM_RES_DWN    Resistive Pull Down 
*  CF_CD_DM_RES_UPDWN  Resistive Pull Up/Down 
*  CF_CD_DM_DIG_HIZ    High Impedance Digital 
*  CF_CD_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void CF_CD_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(CF_CD_0, mode);
}


/*******************************************************************************
* Function Name: CF_CD_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro CF_CD_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 CF_CD_Read(void) 
{
    return (CF_CD_PS & CF_CD_MASK) >> CF_CD_SHIFT;
}


/*******************************************************************************
* Function Name: CF_CD_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 CF_CD_ReadDataReg(void) 
{
    return (CF_CD_DR & CF_CD_MASK) >> CF_CD_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(CF_CD_INTSTAT) 

    /*******************************************************************************
    * Function Name: CF_CD_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 CF_CD_ClearInterrupt(void) 
    {
        return (CF_CD_INTSTAT & CF_CD_MASK) >> CF_CD_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
