/*******************************************************************************
* File Name: EMIF_1_PM.c
* Version 1.30
*
* Description:
*  This file provides Sleep APIs for EMIF Component.
*
* Note:
*   None    
*
********************************************************************************
* Copyright 2011-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "EMIF_1.h"

static EMIF_1_BACKUP_STRUCT EMIF_1_backup;


/*******************************************************************************
* Function Name: EMIF_1_SaveConfig
********************************************************************************
*
* Summary:
*  All configuration registers are retention. Nothing to save here.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void EMIF_1_SaveConfig(void) 
{
    
}


/*******************************************************************************
* Function Name: EMIF_1_Sleep
********************************************************************************
*
* Summary:
*  Stops the EMIF operation and saves the user configuration along with 
*  the enable state of the EMIF.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  EMIF_1_backup - used to save component state before enter sleep 
*  mode.
*
*******************************************************************************/
void EMIF_1_Sleep(void) 
{
    if(EMIF_1_ENABLE_REG == EMIF_1_ENABLE)       
    {
        EMIF_1_backup.enableState = 1u;
    }
    else /* The EMIF block is disabled */
    {
        EMIF_1_backup.enableState = 0u;
    }
    
    EMIF_1_Stop();
    EMIF_1_SaveConfig();
}


/*******************************************************************************
* Function Name: EMIF_1_RestoreConfig
********************************************************************************
*
* Summary:
*  All configuration registers are retention. Nothing to restore here. 
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void EMIF_1_RestoreConfig(void) 
{
    
}


/*******************************************************************************
* Function Name: EMIF_1_Wakeup
********************************************************************************
*
* Summary:
*  Restores the user configuration and restores the enable state.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void EMIF_1_Wakeup(void) 
{
    EMIF_1_RestoreConfig();
    
    if(EMIF_1_backup.enableState != 0u)
    {
        /* Enable component's operation */
        EMIF_1_Enable();
    } /* Do nothing if component's block was disabled before */
}

/* [] END OF FILE */
