/*******************************************************************************
* File Name: REC.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "REC.h"


/*******************************************************************************
* Function Name: REC_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void REC_Write(uint8 value) 
{
    uint8 staticBits = (REC_DR & (uint8)(~REC_MASK));
    REC_DR = staticBits | ((uint8)(value << REC_SHIFT) & REC_MASK);
}


/*******************************************************************************
* Function Name: REC_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  REC_DM_STRONG     Strong Drive 
*  REC_DM_OD_HI      Open Drain, Drives High 
*  REC_DM_OD_LO      Open Drain, Drives Low 
*  REC_DM_RES_UP     Resistive Pull Up 
*  REC_DM_RES_DWN    Resistive Pull Down 
*  REC_DM_RES_UPDWN  Resistive Pull Up/Down 
*  REC_DM_DIG_HIZ    High Impedance Digital 
*  REC_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void REC_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(REC_0, mode);
}


/*******************************************************************************
* Function Name: REC_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro REC_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 REC_Read(void) 
{
    return (REC_PS & REC_MASK) >> REC_SHIFT;
}


/*******************************************************************************
* Function Name: REC_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 REC_ReadDataReg(void) 
{
    return (REC_DR & REC_MASK) >> REC_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(REC_INTSTAT) 

    /*******************************************************************************
    * Function Name: REC_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 REC_ClearInterrupt(void) 
    {
        return (REC_INTSTAT & REC_MASK) >> REC_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
