/*******************************************************************************
* File Name: oen.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "oen.h"


/*******************************************************************************
* Function Name: oen_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void oen_Write(uint8 value) 
{
    uint8 staticBits = (oen_DR & (uint8)(~oen_MASK));
    oen_DR = staticBits | ((uint8)(value << oen_SHIFT) & oen_MASK);
}


/*******************************************************************************
* Function Name: oen_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  oen_DM_STRONG     Strong Drive 
*  oen_DM_OD_HI      Open Drain, Drives High 
*  oen_DM_OD_LO      Open Drain, Drives Low 
*  oen_DM_RES_UP     Resistive Pull Up 
*  oen_DM_RES_DWN    Resistive Pull Down 
*  oen_DM_RES_UPDWN  Resistive Pull Up/Down 
*  oen_DM_DIG_HIZ    High Impedance Digital 
*  oen_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void oen_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(oen_0, mode);
}


/*******************************************************************************
* Function Name: oen_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro oen_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 oen_Read(void) 
{
    return (oen_PS & oen_MASK) >> oen_SHIFT;
}


/*******************************************************************************
* Function Name: oen_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 oen_ReadDataReg(void) 
{
    return (oen_DR & oen_MASK) >> oen_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(oen_INTSTAT) 

    /*******************************************************************************
    * Function Name: oen_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 oen_ClearInterrupt(void) 
    {
        return (oen_INTSTAT & oen_MASK) >> oen_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
