/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>

uint32  addr;
uint8   buf[512];
uint8   status;

#define  I2C_WRITE_XFER_MODE   0
#define  I2C_READ_XFER_MODE    1
#define  I2C_MSTR_NO_ERROR     0

// CF Write
#define  FORCE_CF_REG_LOW      Control_Reg_1_Control = (Control_Reg_1_Control&0xf7) // bit3 = CF Resister Select
#define  FORCE_CF_REG_HIGH     Control_Reg_1_Control = (Control_Reg_1_Control|0x08)
#define  FORCE_CF_RESET_LOW    Control_Reg_1_Control = (Control_Reg_1_Control&0xef) // bit4 = CF RESET
#define  FORCE_CF_RESET_HIGH   Control_Reg_1_Control = (Control_Reg_1_Control|0x10)
#define  FORCE_CF_CS_LOW       Control_Reg_1_Control = (Control_Reg_1_Control&0xdf) // bit5 = CF CS
#define  FORCE_CF_CS_HIGH      Control_Reg_1_Control = (Control_Reg_1_Control|0x20)

#define  PLAY_LED_OFF          Control_Reg_1_Control = (Control_Reg_1_Control&0xfe) // bit0 = PLAY LED
#define  PLAY_LED_ON           Control_Reg_1_Control = (Control_Reg_1_Control|0x01)
#define  REC_LED_OFF           Control_Reg_1_Control = (Control_Reg_1_Control&0xfd) // bit1 = REC LED
#define  REC_LED_ON            Control_Reg_1_Control = (Control_Reg_1_Control|0x02)      
#define  EXEC_LED_OFF          Control_Reg_1_Control = (Control_Reg_1_Control&0xbf) // bit6 = EXEC LED
#define  EXEC_LED_ON           Control_Reg_1_Control = (Control_Reg_1_Control|0x40)      

#define  FORCE_REC_MODE        Control_Reg_1_Control = (Control_Reg_1_Control&0xfb) // bit2 : Low (REC mode)
#define  FORCE_PLAY_MODE       Control_Reg_1_Control = (Control_Reg_1_Control|0x04) // bit2 : High (PLAY mode)

// Switch utilities
#define  READ_SWITCH           sw_in = (~Status_Reg_1_Status)&0x3c   // bit2-5 = PLAY/UP, PLAY/DOWN, PLAY_STOP, REC 
#define  PLAY_UP               0x04
#define  PLAY_DOWN             0x08
#define  PLAY_STOP             0x10
#define  REC                   0x20
#define  PLAY_END              0x40

// CF Read
#define  CHECK_CF_READY        d = Status_Reg_1_Status&0x01   // bit0 = CF Ready Status 
#define  CHECK_CF_CD           d = Status_Reg_1_Status&0x02   // bit1 = CF CD 

#define  CONFIG_OPTION_REG     0x00
#define  CONFIG_STATUS_REG     0x02
#define  PIN_REPLACE_REG       0x04
#define  SOCKET_COPY_REG       0x06

// command block & control block regs

#define CB_DATA    0   // data reg         in/out
#define CB_ERR     1   // error            in    
#define CB_FR      1   // feature reg         out
#define CB_SC      2   // sector count     in/out
#define CB_SN      3   // sector number    in/out
#define CB_CL      4   // cylinder low     in/out
#define CB_CH      5   // cylinder high    in/out
#define CB_DH      6   // device head      in/out
#define CB_STAT    7   // primary status   in    
#define CB_CMD     7   // command             out
#define CB_ASTAT   14  // alternate status in    
#define CB_DC      14  // device control      out
#define CB_DA      15  // device address   in    

// error reg (CB_ERR) bits

#define CB_ER_ICRC 0x80    // ATA Ultra DMA bad CRC
#define CB_ER_BBK  0x80    // ATA bad block
#define CB_ER_UNC  0x40    // ATA uncorrected error
#define CB_ER_MC   0x20    // ATA media change
#define CB_ER_IDNF 0x10    // ATA id not found
#define CB_ER_MCR  0x08    // ATA media change request
#define CB_ER_ABRT 0x04    // ATA command aborted
#define CB_ER_NTK0 0x02    // ATA track 0 not found
#define CB_ER_NDAM 0x01    // ATA address mark not found

// bits 7-4 of the device/head (CB_DH) reg

#define CB_DH_DEV0 0xa0    // select device 0
#define CB_DH_DEV1 0xb0    // select device 1

// status reg (CB_STAT and CB_ASTAT) bits

#define CB_STAT_BSY  0x80  // busy
#define CB_STAT_RDY  0x40  // ready
#define CB_STAT_DF   0x20  // device fault
#define CB_STAT_WFT  0x20  // write fault (old name)
#define CB_STAT_SKC  0x10  // seek complete
#define CB_STAT_SERV 0x10  // service
#define CB_STAT_DRQ  0x08  // data request
#define CB_STAT_CORR 0x04  // corrected
#define CB_STAT_IDX  0x02  // index
#define CB_STAT_ERR  0x01  // error (ATA)
#define CB_STAT_CHK  0x01  // check (ATAPI)

// device control reg (CB_DC) bits

#define CB_DC_HD15   0x08  // bit should always be set to one
#define CB_DC_SRST   0x04  // soft reset
#define CB_DC_NIEN   0x02  // disable interrupts

// Most mandtory and optional ATA commands (from ATA-3),

#define CMD_CFA_ERASE_SECTORS            0xC0
#define CMD_CFA_REQUEST_EXT_ERR_CODE     0x03
#define CMD_CFA_TRANSLATE_SECTOR         0x87
#define CMD_CFA_WRITE_MULTIPLE_WO_ERASE  0xCD
#define CMD_CFA_WRITE_SECTORS_WO_ERASE   0x38
#define CMD_CHECK_POWER_MODE1            0xE5
#define CMD_CHECK_POWER_MODE2            0x98
#define CMD_DEVICE_RESET                 0x08
#define CMD_EXECUTE_DEVICE_DIAGNOSTIC    0x90
#define CMD_FLUSH_CACHE                  0xE7
#define CMD_FORMAT_TRACK                 0x50
#define CMD_IDENTIFY_DEVICE              0xEC
#define CMD_IDENTIFY_DEVICE_PACKET       0xA1
#define CMD_IDENTIFY_PACKET_DEVICE       0xA1
#define CMD_IDLE1                        0xE3
#define CMD_IDLE2                        0x97
#define CMD_IDLE_IMMEDIATE1              0xE1
#define CMD_IDLE_IMMEDIATE2              0x95
#define CMD_INITIALIZE_DRIVE_PARAMETERS  0x91
#define CMD_INITIALIZE_DEVICE_PARAMETERS 0x91
#define CMD_NOP                          0x00
#define CMD_PACKET                       0xA0
#define CMD_READ_BUFFER                  0xE4
#define CMD_READ_DMA                     0xC8
#define CMD_READ_DMA_QUEUED              0xC7
#define CMD_READ_MULTIPLE                0xC4
#define CMD_READ_SECTORS                 0x20
#define CMD_READ_VERIFY_SECTORS          0x40
#define CMD_RECALIBRATE                  0x10
#define CMD_SEEK                         0x70
#define CMD_SET_FEATURES                 0xEF
#define CMD_SET_MULTIPLE_MODE            0xC6
#define CMD_SLEEP1                       0xE6
#define CMD_SLEEP2                       0x99
#define CMD_STANDBY1                     0xE2
#define CMD_STANDBY2                     0x96
#define CMD_STANDBY_IMMEDIATE1           0xE0
#define CMD_STANDBY_IMMEDIATE2           0x94
#define CMD_WRITE_BUFFER                 0xE8
#define CMD_WRITE_DMA                    0xCA
#define CMD_WRITE_DMA_QUEUED             0xCC
#define CMD_WRITE_MULTIPLE               0xC5
#define CMD_WRITE_SECTORS                0x30
#define CMD_WRITE_VERIFY                 0x3C

// TLV320AIC23
#define  ID_TLV320AIC23   0x34;   // I2C device id
#define  BYTE_TLV320      24

static code unsigned char data_tlv320[BYTE_TLV320] = {
   0x1e, 0x00, // Reset Register                         (Address: 0001111)
   0x0c, 0x02, // Power Down Control                     (Address: 0000110)
   0x08, 0x12, // Analog  Audio Path Control             (Address: 0000100) Line, DAC enable, Mic mute
   0x0a, 0x01, // Digital Audio Path Control             (Address: 0000101)

   0x0e, 0x40, // Digitl Audio Interface Format          (Address: 0000111) MSB first, Right align, Rec
   0x0e, 0x42, // Digitl Audio Interface Format          (Address: 0000111) I2S, Play  //MSB first, Left  align, Play

   //0x10, 0x0d, // Sample Rate Control                    (Address: 0001000) MCLK, 8KHz sampling, USB mode
   //0x10, 0xd9, // Sample Rate Control                    (Address: 0001000) MCLK/2, 16KHz sampling, USB mode
   0x10, 0xe3, // Sample Rate Control                    (Address: 0001000) MCLK/2, 22.05KHz sampling, USB mode
   //0x10, 0x23, // Sample Rate Control                    (Address: 0001000) MCLK, 44.1KHz sampling, USB mode
   //0x10, 0xc1, // Sample Rate Control                    (Address: 0001000) MCLK/2, 24KHz sampling, USB mode
   //0x10, 0x01, // Sample Rate Control                    (Address: 0001000) MCLK, 48KHz sampling, USB mode

   0x12, 0x01, // Digital Interface Activation           (Address: 0001001)
   0x01, 0x1d, // Left  line input volume control        (Address: 0000000) Simultaneous update, Normal, +6 dB
   0x03, 0x1d, // Right line input volume control        (Address: 0000001) Simultaneous update, Normal, +6 dB
   0x05, 0xff, // Left  Channel Headphone Volume Control (Address: 0000010) Simultaneous update, Zero-cross detect, +6 dB
   0x07, 0xff, // Right Channel Headphone Volume Control (Address: 0000011) Simultaneous update, Zero-cross detect, +6 dB
};

static code unsigned char wav_header[44] = {
   0x52, 0x49, 0x46, 0x46, // "RIFF"
   0x00, 0x00, 0x00, 0x00, // length of the entire file  - 8 byte
   0x57, 0x41, 0x56, 0x45, // "WAVE"
   0x66, 0x6d, 0x74, 0x20, // "fmt "
   0x10, 0x00, 0x00, 0x00, // 
   0x01, 0x00,             // mono WAV
   0x01, 0x00,             // one channel
   //0x40, 0x1f, 0x00, 0x00, // sample/second 8000=0x001f40
   //0x80, 0x3e, 0x00, 0x00, // bytes/second 16000=0x003e80
   //0x80, 0x3e, 0x00, 0x00, // sample/second 16000=0x003e80
   //0x00, 0x7d, 0x00, 0x00, // bytes/second  32000=0x007d00
   0x22, 0x56, 0x00, 0x00, // sample/second 22050=0x005622
   0x44, 0xac, 0x00, 0x00, // bytes/second  44100=0x00ac44
   //0x44, 0xac, 0x00, 0x00, // sample/second 44100=0x00ac44
   //0x88, 0x58, 0x01, 0x00, // bytes/second  88200=0x015888
   //0x80, 0xbb, 0x00, 0x00, // sample/second 48000=0x00bb80
   //0x00, 0x77, 0x01, 0x00, // bytes/second  96000=0x017700
   0x02, 0x00,             // bytes/sample x channel (2 byte)
   0x10, 0x00,             // bits/sample  (16 bit)
   0x64, 0x61, 0x74, 0x61, // "data"
   0x00, 0x00, 0x00, 0x00, // the number of bytes of data
};

static code unsigned char dat_t[15] = {
  0x20, 0x00, 0x00, 0x93, 0x12, 0xa9, 0x18, 0x88, 0x2f, 0x00,
  0x00, 0x93, 0x12, 0xa9, 0x18,
};
static code unsigned char name_t[11] = {
  'A', 'U', 'D', 'I', 'O', '0', '0', '1',
  'W', 'A', 'V',
};

struct data_entry {
  unsigned char  name[11];
  unsigned char  dat[21];
};

struct data_entry   fd;

uint8   err;
uint8   test_no;
uint8   to_buf;
uint32  lba;
uint32  lba0;
uint8   find;
uint16  fd_next_FAT_byte;
uint16  fd_next_clustor;
uint16  bytes_per_sector;
uint8   sectors_per_clustor;
uint16  reserved_sectors;
uint16  root_entry;
uint32  sectors_per_FAT;
uint32  root_directory;
uint8   root_entry_sectors;
uint32  data_area_start;
uint32  total_sectors;
uint32  BPB_sector;
uint8   FAT_type;
uint8   file_no_search;
uint16  next_file_no;
uint16  max_file_no;
uint8   file_find;
uint8   next_file_direct;
uint32  file_size_byte0;
uint16  file_size_sector;
uint8   on_play;
uint8   audio_mode;
uint8   rec_sample_rate;
uint8   play_sample_rate;
uint8   in_volume;
uint8   out_volume;
uint16  fd_next_FAT_sector;
uint16  fd_first_clustor;
uint32  fd_next_sector;
uint8   from_buf;
uint16  clustor_count;
uint16  sector_count;
uint32  max_sectors;
uint8   play_mono;

void EEPROM_write(void);
void EEPROM_read(void);
void TLV320_rec(void);
void TLV320_play(void);
void TLV320_play_volume(void);
void cf_mode_init(void);
void cf_read_data(void);
void cf_write_data(void);
void set_lba(void);
void BPB_check(void);
void BPB_data(void);
void cf_read_dir(void);
void cf_write_dir(void);
uint8 hold_check(void);
void blink_led(void);
uint8 volume_rate_change(void);
uint8 play_volume_change(void);
void FAT_check(void);
uint8 audio_play(void);
uint8 do_play(void);
uint8 do_rec(void);


void EEPROM_write()
{
    uint8   i, d;
    uint8   err;
       
    //write
    err = 0;
    status = I2C_1_MasterSendStart(0x51, I2C_WRITE_XFER_MODE);
    if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
    {
        for (i=0; i<5; i++)
        {
            switch (i)
            {
                case 0: d = 0x1f; // address1
                        break;
                case 1: d = 0x00; // address2
                        break;
                case 2: d = in_volume;
                        break;
                case 3: d = out_volume;
                        break;
                case 4: d = rec_sample_rate;
                        break;
                default:
                        break;
            }
            status = I2C_1_MasterWriteByte(d);
            if(status != I2C_MSTR_NO_ERROR)
            {
                err = 1;
                break;
            }
        }    
    } 
    else
    {
        err = 1;
    }
    I2C_1_MasterSendStop(); /* Send Stop */
    
    if (err)
    {
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("I2C WRITE ERROR");
    }

}

void EEPROM_read()
{
    uint8   i, d;
    uint8   err;

    //read
    err = 0;
    status = I2C_1_MasterSendStart(0x51, I2C_WRITE_XFER_MODE);
    if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
    {
        /* Send address 2 bytes */
        for(i=0; i<2; i++)
        {
            switch (i)
            {
                case 0: d = 0x1f; //address1
                        break;
                case 1: d = 0x00; //address2
                        break;
                default:
                        break;
            }
            status = I2C_1_MasterWriteByte(d);
            if(status != I2C_MSTR_NO_ERROR)
            {
                err = 1;
                break;
            }
        }
    }
    else
    {
        err= 1;
    }
    I2C_1_MasterSendStop(); /* Send Stop */
        
    if (err)
    {
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("I2C READ ERROR 1");
    }
    
    err = 0;
    status = I2C_1_MasterSendStart(0x51, I2C_READ_XFER_MODE);
    if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
    {
        /* Read 3 bytes */
        for(i=0; i<3; i++)
        {
            switch (i)
            {
                case 0: d = 1; //ACK
                        in_volume = I2C_1_MasterReadByte(d);
                        break;
                case 1: d = 1; //ACK
                        out_volume = I2C_1_MasterReadByte(d);
                        break;
                case 2: d = 0; //NACK
                        rec_sample_rate = I2C_1_MasterReadByte(d);
                        break;
                default:
                        break;
            }
        }
    }
    else
    {
        err= 1;
    }
    I2C_1_MasterSendStop(); /* Send Stop */
    
    if (err)
    {
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("I2C READ ERROR 2");
    }
}

void TLV320_rec()
{
    uint8   i, d;
    uint8   byte_count, data_count;
    uint8   k, j;
       
    //write
    err = 0;
    byte_count = BYTE_TLV320;
    data_count = 3;
    for (i=0; i<byte_count; i=i+2)
    {
        status = I2C_1_MasterSendStart(0x1A, I2C_WRITE_XFER_MODE);
        if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
        {
            k = i;
            if (i==8)
            {
                i += 2;
            }
            for (j=1; j<data_count; j++)
            {
                switch (j)
                {
                    case 1: d = data_tlv320[k];
                            break;
                    case 2: d = data_tlv320[k+1];
                            if (k==12)
                            {
                                if (rec_sample_rate==1) // 16KHz
                                {
                                    d = 0xd9;
                                }
                                if (rec_sample_rate==2) // 8KHz
                                {
                                    d = 0x0d;
                                }
                                if (rec_sample_rate==3) // 44.1KHz
                                {
                                    d = 0x23;
                                }
                                if (rec_sample_rate==4) // 48KHz
                                {
                                    d = 0x01;
                                }                            }
                            if (k==18)
                            {
                                d = in_volume;
                            }
                            break;
                    default:
                            break;
                }
                status = I2C_1_MasterWriteByte(d);
                if(status != I2C_MSTR_NO_ERROR)
                {
                    err = 1;
                    break;
                }
            }    
        }
        else
        {
            err = 1;
            break;
        }
        I2C_1_MasterSendStop(); /* Send Stop */
        
        CyDelay(10);
    }
    if (err)
    {
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("I2C TLV ERROR 0 ");
    }
}

void TLV320_play()
{
    uint8   i, d;
    uint8   byte_count, data_count;
    uint8   k, j;
       
    //write
    err = 0;
    byte_count = BYTE_TLV320;
    data_count = 3;
    for (i=0; i<byte_count; i=i+2)
    {
        status = I2C_1_MasterSendStart(0x1A, I2C_WRITE_XFER_MODE);
        if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
        {
            k = i;
            if (i==8)
            {
                i += 2;
                k = i; //Play mode
            }
            for (j=1; j<data_count; j++)
            {
                switch (j)
                {
                    case 1: d = data_tlv320[k];
                            break;
                    case 2: d = data_tlv320[k+1];
                            if (k==12)
                            {
                                if (play_sample_rate==1) // 16Khz
                                {
                                    d = 0xd9;
                                }
                                if (play_sample_rate==2) // 8KHz
                                {
                                    d = 0x0d;
                                }
                                if (play_sample_rate==3) // 44.1KHz
                                {
                                    d = 0x23;
                                }
                                if (play_sample_rate==4) // 48KHz
                                {
                                    d = 0x01;
                                }
                            }
                            if (k==22)
                            {
                                d = (out_volume|0x80);
                            }
                            break;
                    default:
                            break;
                }
                status = I2C_1_MasterWriteByte(d);
                if(status != I2C_MSTR_NO_ERROR)
                {
                    err = 1;
                    break;
                }
            }    
        }
        else
        {
            err = 1;
            break;
        }
        I2C_1_MasterSendStop(); /* Send Stop */
        
        CyDelay(10);
    }
    if (err)
    {
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("I2C TLV ERROR 1 ");
    }
}

void TLV320_play_volume()
{
    uint8   j, d;
    uint8   byte_count, data_count;
       
    //write
    err = 0;
    byte_count = 1; // out_volume set
    data_count = 3; 

    status = I2C_1_MasterSendStart(0x1A, I2C_WRITE_XFER_MODE);
    if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
    {
        for (j=1; j<data_count; j++)
        {
            switch (j)
            {
                case 1: d = data_tlv320[22];
                        break;
                case 2: d = (out_volume|0x80);
                        break;
                default:
                        break;
            }
            status = I2C_1_MasterWriteByte(d);
            if(status != I2C_MSTR_NO_ERROR)
            {
                err = 1;
                break;
            }
        }    
    }
    else
    {
        err = 1;
    }
    I2C_1_MasterSendStop(); /* Send Stop */
        
    CyDelay(10);

    status = I2C_1_MasterSendStart(0x1A, I2C_WRITE_XFER_MODE);
    if(I2C_MSTR_NO_ERROR == status) /* Check if transfer completed without errors */
    {
        for (j=1; j<data_count; j++)
        {
            switch (j)
            {
                case 1: d = data_tlv320[20];
                        break;
                case 2: d = (out_volume|0x80);
                        break;
                default:
                        break;
            }
            status = I2C_1_MasterWriteByte(d);
            if(status != I2C_MSTR_NO_ERROR)
            {
                err = 1;
                break;
            }
        }    
    }
    else
    {
        err = 1;
    }
    I2C_1_MasterSendStop(); /* Send Stop */
        
    CyDelay(10);
    
    if (err)
    {
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("I2C TLV ERROR 2 ");
    }
}

void cf_mode_init()
{
    uint8   d;
    
    FORCE_CF_CS_HIGH;
    
    FORCE_CF_REG_LOW;
    CyDelay(10);
    
    FORCE_CF_RESET_HIGH;
    CyDelay(10);
    FORCE_CF_RESET_LOW;
     
    do {
      CHECK_CF_READY;
      CyDelay(10);
    } while (!d);
    
    FORCE_CF_REG_LOW;  //attribute area select
    CyDelay(100);
    
    FORCE_CF_CS_LOW;
    
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)SOCKET_COPY_REG;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CONFIG_OPTION_REG;
    CY_SET_XTND_REG8(addr, 0);
    /*
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CONFIG_STATUS_REG;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)PIN_REPLACE_REG;
    CY_SET_XTND_REG8(addr, 0);
    FORCE_CF_CS_HIGH;
    */
   
    FORCE_CF_REG_HIGH;  //task area select
        
    // Device Initialize
    
    //Execute a Software Reset
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DC;
    CY_SET_XTND_REG8(addr, /* CB_DC_HD15 | */ CB_DC_NIEN | CB_DC_SRST);
    CyDelay(1);
    CY_SET_XTND_REG8(addr, /* CB_DC_HD15 | */ CB_DC_NIEN);
    CyDelay(1);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & CB_STAT_BSY ) == 0 )
            break;
        CyDelay(10);
    }
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_FR;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SC;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SN;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CL;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CH;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DH;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & CB_STAT_BSY ) == 0 )
            break;
        CyDelay(1);
    }
    
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_FR;
    CY_SET_XTND_REG8(addr, 0x03); // Byte Access
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CMD;
    CY_SET_XTND_REG8(addr, 0xEF); // Set Feature
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & CB_STAT_BSY ) == 0 )
            break;
        CyDelay(1);
    }

    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CMD;
    CY_SET_XTND_REG8(addr, CMD_IDLE1);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & CB_STAT_BSY ) == 0 )
            break;
        CyDelay(1);
    }
    FORCE_CF_CS_HIGH; 
}

void cf_read_data()
{
    uint8   i;
    uint16  j;
    uint8   d;
    
    set_lba();
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CMD;
    CY_SET_XTND_REG8(addr, CMD_READ_SECTORS);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_ASTAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & ( CB_STAT_BSY | CB_STAT_DRQ ) ) == CB_STAT_DRQ )
            break;
        CyDelay(1);
    }
    
    i=0;
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    status = CY_GET_XTND_REG8(addr);
    if ( ( status & ( CB_STAT_BSY | CB_STAT_DRQ ) ) == CB_STAT_DRQ )
    {
        status = 0;
        addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DATA;
        for ( j=0; j<512; j++ )
        {
            d = CY_GET_XTND_REG8(addr);
            
            if (to_buf)
                buf[j] = d;

            if ((test_no==7)&&(!find)) // FAT check
            {
                if ((j&0x01)&&(!i)&&(!d))
                {
                    status ++;
                    if (status==1)
                    {
                        fd_next_FAT_byte = (j-1);
                        fd_next_clustor += (fd_next_FAT_byte>>1);
                    }
                    if (status==3)
                    find = 1;
                }
                else
                    i = d;
                if (status&&d) status = 0;
            }
        }
    }
    
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    status = CY_GET_XTND_REG8(addr);
}

void cf_write_data()
{
    uint8   d, status;
    uint16  j;

    set_lba();
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CMD;
    CY_SET_XTND_REG8(addr, CMD_WRITE_SECTORS);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_ASTAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & ( CB_STAT_BSY | CB_STAT_DRQ ) ) == CB_STAT_DRQ )
            break;
        CyDelay(1);
    }
    
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    status = CY_GET_XTND_REG8(addr);
    if ( ( status & ( CB_STAT_BSY | CB_STAT_DRQ ) ) == CB_STAT_DRQ )
    {
        status = 0;
        addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DATA;
        for ( j=0; j<512; j++ )
        {
            if (from_buf)
                d = buf[j];
            else
            {
                if (j<44)
                {
                    d = wav_header[j];
	                if (rec_sample_rate==1) // 16KHz
	                {
	                    if (j==24) d = 0x80;
	                    if (j==25) d = 0x3e;
		                if (j==28) d = 0x00;
		                if (j==29) d = 0x7d;
	                }
	                if (rec_sample_rate==2) // 8KHz
	                {
	                    if (j==24) d = 0x40;
	                    if (j==25) d = 0x1f;
		                if (j==28) d = 0x80;
		                if (j==29) d = 0x3e;
	                }
                    if (rec_sample_rate==3) // 44.1KHz
	                {
	                    if (j==24) d = 0x44;
	                    if (j==25) d = 0xac;
		                if (j==28) d = 0x88;
		                if (j==29) d = 0x58;
                        if (j==30) d = 0x01;
	                }
                }
                else
                    d = 0; 
            }
            CY_SET_XTND_REG8(addr, d);
        }
    }
    CyDelay(100);
    
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
    while(1)
    {
        if ( ( CY_GET_XTND_REG8(addr) & CB_STAT_BSY ) == 0 )
            break;
        CyDelay(1);
    }
    status = CY_GET_XTND_REG8(addr);
}

void set_lba()
{
    union 
    {
        uint32 lba1;
        uint8  lba2[4];
    } u;

    u.lba1 = lba;

    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DC;
    CY_SET_XTND_REG8(addr, CB_DC_HD15 | CB_DC_NIEN);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_FR;
    CY_SET_XTND_REG8(addr, 0);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SC;
    CY_SET_XTND_REG8(addr, 1);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SN;
    CY_SET_XTND_REG8(addr, u.lba2[3]);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CL;
    CY_SET_XTND_REG8(addr, u.lba2[2]);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CH;
    CY_SET_XTND_REG8(addr, u.lba2[1]);
    addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DH;
    CY_SET_XTND_REG8(addr, 0xE0 | u.lba2[0] ); //LBA mode    
}

void BPB_check()
{
    LCD_Char_Position(0,0);
    LCD_Char_PrintString("                ");
    LCD_Char_Position(1,0);
    LCD_Char_PrintString("                ");
    
    if ((buf[0x39]==0x31)&&(buf[0x3a]==0x36)) /* FAT16 */
    {
        FAT_type = 1;
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("FAT16");
    }
    else if ((buf[0x55]==0x33)&&(buf[0x56]==0x32)) /* FAT32 */
    {
        FAT_type = 2;
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("FAT32");
    }
    else
    {
        FAT_type = 0;
        LCD_Char_Position(0,0);
        LCD_Char_PrintString("MBR");
    }
    LCD_Char_Position(1,0);
    LCD_Char_PrintString("BPB lba ");
    LCD_Char_PrintInt32(lba);
}
    
void BPB_data()
{
    uint8  i;
    
    bytes_per_sector    = (((uint16)(buf[12]))<<8)|((uint16)(buf[11]));
    sectors_per_clustor = buf[13];
    reserved_sectors    = (((uint16)(buf[15]))<<8)|((uint16)(buf[14]));

    if (FAT_type==2) /* FAT32 */
    {
        sectors_per_FAT     = (((uint32)(buf[39]))<<24)|(((uint32)(buf[38]))<<16)|(((uint32)(buf[37]))<<8)|((uint32)(buf[36]));
        root_directory      = (uint32)reserved_sectors;
        for (i=0; i<buf[16]; i++)
            root_directory += sectors_per_FAT;

        root_entry          = (((uint16)(buf[18]))<<8)|((uint16)(buf[17]));
        root_entry_sectors  = (uint8)(root_entry>>4);
        data_area_start     = root_directory;
   }
   else  /* FAT16 */
   {
        sectors_per_FAT     = (((uint32)(buf[23]))<<8)|((uint32)(buf[22]));
        root_directory      = (uint32)reserved_sectors;
        for (i=0; i<buf[16]; i++)
            root_directory += sectors_per_FAT;

        root_entry          = (((uint16)(buf[18]))<<8)|((uint16)(buf[17]));
        root_entry_sectors  = (uint8)(root_entry>>4);
        data_area_start     = (uint32)root_entry_sectors;
        data_area_start    += root_directory;
    }
}

void cf_read_dir(void)
{
    uint8   ok, i;
    uint32  file_dir_sector, file_dir_sector0, p;
    uint8   file_dir_sector_count, sector_count;
    uint16  file_dir_pos;
    uint16  FAT_pos;
    uint32  FAT_table_next, FAT_table_count;

    err = 0;
    ok  = 0;

    if (FAT_type==1) // FAT16
    {
        if (next_file_no>root_entry)
        {
            err = 1;
            LCD_Char_Position(0,0);
            LCD_Char_PrintString("Error 2");
            CyDelay(5000);
            return ;
        }
    }

    if (!next_file_no)
        err = 1;
    else
    {
        file_dir_sector0      = 0;

        FAT_table_count       = 0;
        FAT_pos               = 2; /* init : root directory */

        sector_count          = sectors_per_clustor;

        do
        {
            file_dir_sector_count = 1;
            file_dir_pos = ((uint16)(next_file_no-1)<<5); // x32
            for (file_dir_sector = 1; file_dir_pos>511; file_dir_sector++)
            {
                file_dir_sector_count ++;
                if (FAT_type==1) /* FAT16 */
                {
                    file_dir_pos -= 512;
                }
                else /* FAT32 */
                {
                    file_dir_pos -= 512;
                    if (file_dir_sector_count > sector_count)
                    {
                        sector_count += sectors_per_clustor;
                  
                        /* next FAT check */
                        test_no =9;
                        lba = (uint32)reserved_sectors + FAT_table_count + BPB_sector;
                        to_buf = 1;
                        cf_read_data();

                        FAT_pos = (FAT_pos << 2); /* x4 */
                        if ((buf[FAT_pos]==0xFF)&&(buf[FAT_pos+1]==0xFF)&&(buf[FAT_pos+2]==0xFF)&&(buf[FAT_pos+3]=0x0F))
                        {
                            file_dir_sector --;
                            if (file_no_search)
                            {
                                max_file_no = next_file_no; /* when no_file, max_file_no=1 */
                                file_no_search = 0;
                            }
                            ok = 1;
                        }
                        else
                        {
                            file_dir_sector = (((uint32)(buf[FAT_pos]))<<24)|(((uint32)(buf[FAT_pos+2]))<<16)|(((uint32)(buf[FAT_pos+1]))<<8)|((uint32)(buf[FAT_pos]));
                     
                            FAT_table_next = file_dir_sector -1;
                            FAT_table_count = (FAT_table_next >> 7); /* 1/128 */
                            while (FAT_table_next > 128)
                            {
                                FAT_table_next -= 128;
                            }
                            FAT_pos = (uint16)FAT_table_next;
                            FAT_pos --;
                     
                            file_dir_sector -= 2;
                            p = 0;
                            for (i=0; i<sectors_per_clustor; i++)
                            {
                                p += file_dir_sector;
                            }
                            file_dir_sector = p;
                            file_dir_sector ++;
                        }
                    }
                }
            }

            /* directory read */
            if ((uint32)file_dir_sector_count != file_dir_sector0)
            {
                file_dir_sector0 = (uint32)file_dir_sector_count;
                lba = root_directory + file_dir_sector + BPB_sector - 1;
                test_no = 8;
                to_buf = 1;
                cf_read_data();
            }
            if ((buf[file_dir_pos+8]=='W')&&(buf[file_dir_pos+9]=='A')&&(buf[file_dir_pos+10]=='V'))
            {
                if (!file_no_search) ok = 1;
                file_find = 1;
            }
            else
            {
                /* Upper limit check */
                if ((!buf[file_dir_pos])&&(!next_file_direct))
                {
                    if (file_no_search)
                    {
                        max_file_no = next_file_no; /* when no_file, max_file_no=1 */
                        file_no_search = 0;
                        ok = 1;
                    }
                    else
                    {
                        if (!file_find)
                        {
                            err = 1;
                        }
                    }
                    next_file_no = 0;  /* normal(loop) */
                }

                /* Lower limit check */
                if ((next_file_no==1)&&next_file_direct)
                {
                    next_file_no = max_file_no;
                }
            }

            /* for search */
            if (!ok)
            {
                if (next_file_direct)
                    next_file_no --;
                else
                    next_file_no ++;

                if (!next_file_no)
			    {
			        err = 1;
			    }
            }
        } while ((!ok)&&(!err));

        if (!err)
        {
            fd_next_clustor  = ((uint16)buf[file_dir_pos + 27]<<8);
            fd_next_clustor += (uint16)buf[file_dir_pos + 26];
            file_size_byte0  = (uint32)buf[file_dir_pos + 31];
            for (i=3; i>0; i--)
            {
                file_size_byte0  = (file_size_byte0<<8);
                file_size_byte0 += (uint32)(buf[file_dir_pos + 27 + i]);
            }
            file_size_sector = (uint16)(file_size_byte0>>9); /* 1/512 */
        }
    }
}

void cf_write_dir()
{
    uint8   i, find_dir_entry, file_dir_sector;
    uint16  j, k;
    uint8   *p;

    find_dir_entry = 0;

    test_no = 8; // directory entry

    // FAT write
    to_buf   = 1;
    from_buf = 1;
    lba = (uint32)reserved_sectors + (uint32)fd_next_FAT_sector + BPB_sector;

    cf_read_data();
    fd_next_clustor ++;
    for (j=1; j<clustor_count; j++)
    {
        buf[fd_next_FAT_byte] = (uint8)(fd_next_clustor&0xff);
        fd_next_FAT_byte ++;
        buf[fd_next_FAT_byte] = (uint8)((fd_next_clustor>>8)&0xff);
        fd_next_FAT_byte ++;

        if (fd_next_FAT_byte==512)
        {
            cf_write_data();
            lba ++;
            cf_read_data();
            fd_next_FAT_byte = 0;
        }
        fd_next_clustor ++;
    }
    buf[fd_next_FAT_byte] = 0xff;
    fd_next_FAT_byte ++;
    buf[fd_next_FAT_byte] = 0xff;

    cf_write_data();

    to_buf = 1;
    lba = root_directory + BPB_sector - 1;

    k = 0;
    file_dir_sector = root_entry_sectors;
    do
    {
        lba ++;

        cf_read_data();
        file_dir_sector --;

        for (j=0; j<512; j+=32)
        {
            k ++;
            if (!buf[j])
            {
                find_dir_entry = 1;

                for (i=0; i<11; i++)
                    fd.name[i] = name_t[i];

                if (k>99)
                    return;

                if (k>9)
                {
                    i = 0;
                    do
                    {
                        i ++;
                        k -= 10;
                    } while (k>9);
                    fd.name[6] = '0'+ i;
                }
                fd.name[7] = '0'+ k;

                for (i=0; i<15; i++)
                    fd.dat[i] = dat_t[i];

                fd.dat[15] = (uint8)(fd_first_clustor&0xff);
                fd.dat[16] = (uint8)((fd_first_clustor>>8)&0xff);
                for (i=0; i<4; i++)
                {
                    fd.dat[17+i]    = (uint8)(file_size_byte0&0xff);
                    file_size_byte0 = (file_size_byte0>>8);
                }

                p = (uint8 *)&fd;
                for (i=0; i<32; i++)
                {
                    buf[j+(unsigned int)i] = (*p);
                    p ++;
                }
                break;
            }
        }
    } while (file_dir_sector&&(!find_dir_entry));

    // directory write
    if (find_dir_entry)
    {
        from_buf = 1;
        cf_write_data();
    }
}

uint8 hold_check(void)
{
    uint8  sw_in, no_hold;

    no_hold = 1;
    READ_SWITCH;
    while(sw_in&&no_hold)
    {
        READ_SWITCH;
        CyDelay(10);
        no_hold ++;
    }
    return (no_hold);
}

void blink_led()
{
     CyDelay(250);
     EXEC_LED_OFF;
     CyDelay(250);
     EXEC_LED_ON;
}

uint8 volume_rate_change()
{
    uint8   sw_in, sw_data, d, ok;

    ok = 0;

    READ_SWITCH;
    while(sw_in)  // wait for switch release
    {
        blink_led();

        READ_SWITCH;
    }

    if (rec_sample_rate==0)  // 22.1KHz
    {
        REC_LED_OFF;
        PLAY_LED_OFF;
        EXEC_LED_ON;
    }
    if (rec_sample_rate==1)  // 16KHz
    {
        REC_LED_OFF;
        PLAY_LED_ON;
        EXEC_LED_OFF;
    }
    if (rec_sample_rate==2)  // 8KHz
    {
        REC_LED_ON;
        PLAY_LED_OFF;
        EXEC_LED_OFF;
    }
    if (rec_sample_rate==3)  // 44.1KHz
    {
        REC_LED_ON;
        PLAY_LED_ON;
        EXEC_LED_ON;
    }

    CyDelay(200);

    do
    {
        READ_SWITCH;

        if (sw_in)
        {
            sw_data = sw_in;

            READ_SWITCH;
            while(sw_in==REC)
            {
                READ_SWITCH;
            }

            if (sw_data != PLAY_STOP)
            {
                if (sw_data==REC)
                {
	                rec_sample_rate ++;
	                if (rec_sample_rate==4) rec_sample_rate = 0;
	            }
                else
    	        {
  	                d = in_volume;
	                if (sw_data==PLAY_UP) // +1.5 dB step
	                {
	                    d ++;
	                    if (d>31) d = 31;
	                }
	                else // -1.5 dB step
	                {
	                    if (d) d --;
	                }
	                in_volume = d;
	            }

                TLV320_rec();  // REC mode

                if (rec_sample_rate==0)  // 22.1KHz
                {
                    REC_LED_OFF;
                    PLAY_LED_OFF;
                    EXEC_LED_ON;
                }
                if (rec_sample_rate==1)  // 16KHz
                {
                    REC_LED_OFF;
                    PLAY_LED_ON;
                    EXEC_LED_OFF;
                }
                if (rec_sample_rate==2)  // 8KHz
                {
                    REC_LED_ON;
                    PLAY_LED_OFF;
                    EXEC_LED_OFF;
                }
                if (rec_sample_rate==3)  // 44.1KHz
                {
                    REC_LED_ON;
                    PLAY_LED_ON;
                    EXEC_LED_ON;
                }
            }
            else
            {
                REC_LED_OFF;
                PLAY_LED_OFF;
                EXEC_LED_OFF;                

                ok = 1;
            }
        }

    } while (!ok);

    EEPROM_write();  // write

    return (sw_in); // PLAY_STOP
}

uint8 play_volume_change()
{
    uint8   sw_in, d, ok;

    ok = 0;

    READ_SWITCH;
    while(sw_in)  // wait for switch release
    {
        blink_led();

        READ_SWITCH;
    }

    do
    {
        while (!sw_in)
        {
            blink_led(); // for delay after switch release

            READ_SWITCH;
        }

        if (sw_in != PLAY_STOP)
        {
            if ((sw_in==PLAY_UP)||(sw_in==PLAY_DOWN))
            {
	            d = out_volume;
	            if (sw_in==PLAY_UP) // +1 dB step
	            {
	                d ++;
  	                if (d>127) d = 127;
	            }
	            else // -1 dB step
	            {
	                if (d) d --;
	            }
	            out_volume = d;
	        }

            audio_mode = 1;  // Play mode
            d = out_volume;
            out_volume = 0;  // mute
            TLV320_play_volume();
            out_volume = d;
            TLV320_play_volume();
            FORCE_PLAY_MODE;

            sw_in = audio_play();
            
            CyDelay(200);
        }
        else
        {
            ok = 1;
        }
        
    } while (!ok);

    EEPROM_write();  // write

    // play current position
    next_file_no ++;
    sw_in = PLAY_DOWN;

    return (sw_in);
}

void FAT_check()
{
    uint16   i;

    test_no = 7; // FAT check
    find = 0;
    lba = reserved_sectors + BPB_sector;

    fd_next_clustor  = 0;
    to_buf = 1;
    for (i=0; i<sectors_per_FAT; i++)
    {
        cf_read_data();
        if (find)
        {
            fd_next_FAT_sector = i;
            fd_first_clustor = fd_next_clustor; //for directory entry
            break;
        }
        else
        {
            lba += 1;
            fd_next_clustor += 256;
        }
    }
    if (!find) err = 1;
}

uint8 audio_play()
{
    uint8    i, mode, j;
    uint8    sw_in;
    uint16   k;

    mode        = audio_mode;
    max_sectors = total_sectors;

    EXEC_LED_ON;

    sw_in     = 0;

    if (mode) // Play
    {
        cf_read_dir(); // set fd_next_clustor, file_size_sector
        sector_count = file_size_sector;
    }
    else // Rec
    {
        FAT_check();          // set fd_next_clustor
        file_size_sector = 1; // not used
    }

    if (err)
        return  PLAY_STOP;

    fd_next_sector = data_area_start;
    for (i=0; i<sectors_per_clustor; i++)
    {
        fd_next_sector += (uint32)fd_next_clustor;
        fd_next_sector -= 2;
    }
    lba = fd_next_sector + BPB_sector;

    lba0 = lba;

    if (mode) // Play
    {
        test_no = 5;
        to_buf = 1;
        cf_read_data();     // wav_header read
        i = buf[24];
        j = buf[25];

        play_mono = 1;
        if (i==0x22)
        {
            play_sample_rate = 0;   // 22.05KHz
            if (buf[29]==0x58) play_mono = 0; // stereo
        }
        if (i==0x80)
        {
            play_sample_rate = 1;   // 16KHz
            if (buf[29]==0xfa) play_mono = 0; // stereo
        }
        if (i==0x40)
        {
            play_sample_rate = 2;   // 8KHx
            if (buf[29]==0x7d) play_mono = 0; // stereo
        }
        if (i==0x44) play_sample_rate = 3;   // 44.1KHz
        if (j==0xBB) play_sample_rate = 4;   // 48KHz

        TLV320_play(); // Play
        
        sector_count --;
    }
    else // Rec
    {
        REC_LED_ON;

        from_buf = 0;
        cf_write_data();   // wav_header write
    }
    
    for (k=0; k<512; k++)
        buf[k] = 0;
    
    lba ++;

    if (mode) //Play
        sw_in = do_play();
    else
        sw_in = do_rec();
        
    return (sw_in);
}

uint8 do_play()
{
    uint8    charactor, lrc;
    uint8    in_count, out_count;
    uint16   data16;
    uint8    status, cf_state;
    uint8    sw_in;
    
    union 
    {
        uint32 lba1;
        uint8  lba2[4];
    } u;
    
    charactor = 0;
    lrc       = 0;
    in_count  = 0;
    out_count = 0;
    cf_state  = 0;    
    
    u.lba1    = lba;
    sw_in     = 0;
    if (!err)
    {
        charactor = (LRC_PS & LRC_MASK);
        lrc = charactor;

        do
        {           
            charactor = (LRC_PS & LRC_MASK);
            if (charactor&&(!lrc)) 
            {
                data16 = ((((uint16)buf[out_count+1])<<8)|((uint16)buf[out_count]));
                CY_SET_REG16(ShiftReg_1_IN_FIFO_VAL_LSB_PTR, data16); 
 		        out_count += 2;
            }
            if (!play_mono) //stereo
            {
                if (!charactor&&(lrc)) 
                {
                    data16 = ((((uint16)buf[out_count+1])<<8)|((uint16)buf[out_count]));
                    CY_SET_REG16(ShiftReg_2_IN_FIFO_VAL_LSB_PTR, data16); 
 		            out_count += 2;
                }
            }
            lrc = charactor;

            switch (cf_state)
            {
                case 0: // idle state1
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DC;
                     CY_SET_XTND_REG8(addr, CB_DC_HD15 | CB_DC_NIEN);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_FR;
                     CY_SET_XTND_REG8(addr, 0);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SC;
                     CY_SET_XTND_REG8(addr, 1);
		             cf_state = 1;
		             break;
	            case 1: // idle state2
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SN;
                     CY_SET_XTND_REG8(addr, u.lba2[3]);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CL;
                     CY_SET_XTND_REG8(addr, u.lba2[2]);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CH;
                     CY_SET_XTND_REG8(addr, u.lba2[1]);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DH;
                     CY_SET_XTND_REG8(addr, 0xE0 | u.lba2[0] ); //LBA mode    
                     cf_state = 2;
		             break;
                case 2: // command state
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CMD;
                     CY_SET_XTND_REG8(addr, CMD_READ_SECTORS);
                     cf_state = 3;
		             break;
                case 3: // ready?
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_ASTAT;
                     status = CY_GET_XTND_REG8(addr);
                     if ( ( status & CB_STAT_BSY ) == 0 )
                         cf_state = 4;
		             break;
                case 4: // ready?
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
                     status = CY_GET_XTND_REG8(addr);
                     if ( ( status & ( CB_STAT_BSY | CB_STAT_DRQ ) ) == CB_STAT_DRQ )
                     {
                         addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DATA;
                         cf_state = 5;
                     }
		             break;
                case 5: // data read/write1
                     if ((in_count+2)!=out_count)
			         {
                         buf[in_count] = cyread8((volatile void far *)addr);
                         in_count ++;
                         buf[in_count] = cyread8((volatile void far *)addr);
                         in_count ++;

			             if (!in_count) cf_state = 6;
			         }
		             break;
                case 6: // data read/write2
                     if ((in_count+2)!=out_count)
			         {
                         buf[in_count] = cyread8((volatile void far *)addr);
                         in_count ++;
                         buf[in_count] = cyread8((volatile void far *)addr);
                         in_count ++;

			             if (!in_count) cf_state = 7;
			         }
 		             break;
                case 7: // ready?
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_ASTAT;
                     status = CY_GET_XTND_REG8(addr);
                     if ( ( status & CB_STAT_BSY ) == 0 )
                         cf_state = 8;
		             break;
                case 8: // status read
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
                     status = CY_GET_XTND_REG8(addr);
                     cf_state = 9;
		             break;
                case 9: // lba increment
                     u.lba1 ++;
                     cf_state = 10;
                     READ_SWITCH;
		             break;
	            case 10:// end check
                     cf_state = 0;
                     sector_count --;
                     if (sector_count==0)
                         sw_in = PLAY_END;
		             break;
                default:
                     break;
            }
        } while (!sw_in); // end of while

        EXEC_LED_OFF;

        if (sw_in==PLAY_END) sw_in = 0;
    }

    return sw_in;
}

uint8 do_rec()
{
    uint8    i, charactor, lrc;
    uint8    in_count, out_count;
    uint16   data16;
    uint8    cf_state;
    uint8    sw_in;
    
    union 
    {
        uint32 lba1;
        uint8  lba2[4];
    } u;
    
    charactor = 0;
    lrc       = 0;
    in_count  = 0;
    out_count = 0;
    cf_state  = 0;    
    
    u.lba1    = lba;
    sw_in     = 0;
    
    i = 0;
    if (!err)
    {
        charactor = (LRC_PS & LRC_MASK);
        lrc = charactor;

        do
        {           
            charactor = (LRC_PS & LRC_MASK);
            if (charactor&&(!lrc)) 
            {
                data16 = CY_GET_REG16(ShiftReg_1_OUT_FIFO_VAL_LSB_PTR); 
                buf[in_count] =  (uint8)data16; // low 8 bit
                buf[in_count+1] =  (uint8)((data16>>8)); // high 8 bit
                in_count +=2;
            }
            lrc = charactor;

            switch (cf_state)
            {
                case 0: // idle state1
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DC;
                     CY_SET_XTND_REG8(addr, CB_DC_HD15 | CB_DC_NIEN);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_FR;
                     CY_SET_XTND_REG8(addr, 0);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SC;
                     CY_SET_XTND_REG8(addr, 1);
		             cf_state = 1;
		             break;
	            case 1: // idle state2
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_SN;
                     CY_SET_XTND_REG8(addr, u.lba2[3]);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CL;
                     CY_SET_XTND_REG8(addr, u.lba2[2]);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CH;
                     CY_SET_XTND_REG8(addr, u.lba2[1]);
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DH;
                     CY_SET_XTND_REG8(addr, 0xE0 | u.lba2[0] ); //LBA mode    
                     cf_state = 2;
		             break;
                case 2: // command state
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_CMD;
                     CY_SET_XTND_REG8(addr, CMD_WRITE_SECTORS);
                     cf_state = 3;
		             break;
                case 3: // ready?
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_ASTAT;
                     status = CY_GET_XTND_REG8(addr);
                     if ( ( status & CB_STAT_BSY ) == 0 )
                         cf_state = 4;
		             break;
                case 4: // ready?
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
                     status = CY_GET_XTND_REG8(addr);
                     if ( ( status & ( CB_STAT_BSY | CB_STAT_DRQ ) ) == CB_STAT_DRQ )
                     {
                         addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_DATA;
                         cf_state = 5;
                     }
		             break;
                case 5: // data read/write1
                     if (in_count!=out_count)
                     {
                         cywrite8((volatile void far*)addr, buf[out_count]); 
                         cywrite8((volatile void far*)addr, buf[out_count+1]); 
                         out_count += 2;
                            
			             if (!out_count) cf_state = 6;
                     }
		             break;
                case 6: // data read/write2
                     if (in_count!=out_count)
                     {
                         cywrite8((volatile void far*)addr, buf[out_count]); 
                         cywrite8((volatile void far*)addr, buf[out_count+1]); 
                         out_count += 2;

			             if (!out_count) cf_state = 7;
			         }
 		             break;
                case 7: // ready?
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_ASTAT;
                     status = CY_GET_XTND_REG8(addr);
                     if ( ( status & CB_STAT_BSY ) == 0 )
                         cf_state = 8;
		             break;
                case 8: // status read
                     addr = (uint32)CYDEV_EXTMEM_BASE + (uint32)CB_STAT;
                     status = CY_GET_XTND_REG8(addr);
                     cf_state = 9;
		             break;
                case 9: // lba increment
                     u.lba1 ++;
                     cf_state = 10;
                     READ_SWITCH;
		             break;
	            case 10:// end check
                     cf_state = 0;
                     if (u.lba1==max_sectors)
                         sw_in = PLAY_STOP;
		             break;
                default:
                     break;
            }
        } while (!sw_in); // end of while

        EXEC_LED_OFF;

        REC_LED_OFF;

        lba = u.lba1;

        lba = lba - lba0;
        file_size_byte0 = lba*512;
        u.lba1 = file_size_byte0 -1;

        i = (uint8)lba;
        i &= (sectors_per_clustor-1);
        if (i)
            clustor_count = 1;
        else
            clustor_count = 0;
        i = sectors_per_clustor;
        do
        {
            lba = (lba>>1);
            i = (i>>1);
        } while (i>1);
        clustor_count  += (uint16)(lba);
        cf_write_dir();

        test_no = 5;
        to_buf = 1;
        lba = fd_next_sector + BPB_sector;
        cf_read_data();

        buf[4] = 0xf8;
        buf[5] = u.lba2[2];
        buf[6] = u.lba2[1];
        buf[7] = u.lba2[0];
        buf[40] = 0xd4;
        buf[41] = buf[5];
        buf[42] = buf[6];
        buf[43] = buf[7];

        from_buf = 1;
        cf_write_data();
    }

    return sw_in;
}

int main()
{
    uint8   sw_in, sw_data;
    uint8   play_mode;
    uint8   no_hold;
        
    LCD_Char_Start();
    
    err = 0;
    I2C_1_Start();
    
    I2C_1_MasterClearStatus();
    
    EEPROM_read();
    
    if (in_volume==0xff)
    {
       in_volume        = 0x1d;
       out_volume       = 0x7f;
    }
    
    rec_sample_rate = 3; // 0:22.05KHz 1:16KHz 2:8KHz 3:44.1KHz
    
    TLV320_rec();

    Control_Reg_1_Write(0x38); //Init [6:0]: EXEC_LED,CF_CS,CF_RESET,CF_REG,AUDIO_MODE,REC_LED,PLAY_LED
    
    EMIF_1_Start();
    
    ShiftReg_1_Start();
    ShiftReg_2_Start();
    
    cf_mode_init();
    
    FORCE_CF_CS_LOW;
    
    test_no = 0; // Master Boot Record read
    lba = 0;
	to_buf = 1;
    cf_read_data();    
    BPB_check();
    CyDelay(1000);
    
    BPB_sector = 0;
    if (!FAT_type)
    {
       BPB_sector = (((uint32)(buf[457]))<<24)|(((uint32)(buf[456]))<<16)|(((uint32)(buf[455]))<<8)|((uint32)(buf[454]));
       test_no = 1; // BIOS Parameter Block read
       lba = BPB_sector; //0x3F;  // 0x20; //BPB_sector;
           
       to_buf = 1;
       cf_read_data();  
    
       BPB_check();
    
       if (!FAT_type)
       {
           err = 1;
           LCD_Char_Position(0,0);
           LCD_Char_PrintString("Error 1");
       }
    }
    CyDelay(10);
    
    BPB_data();
    
    lba = root_directory + BPB_sector;
    to_buf = 1;
    cf_read_data();
    
    file_no_search   = 1;
    next_file_no     = 1; /* for max_file_no search */

    cf_read_dir();      /* set max_file_no; next_file_no = 0 */
    
    next_file_direct = 0;
    next_file_no     = 1;
    
    //Player start
    audio_mode       = 0; //Rec mode
    on_play          = 0;
    play_sample_rate = 0;
    play_mode        = 0; //0:normal(loop) 1:repeate
    no_hold          = 1;
    
    FORCE_REC_MODE;
    while (!err)
    {
        READ_SWITCH;

        if (sw_in)
        {
            sw_data = sw_in;
 /*  
            LCD_Char_Position(0,0);
            LCD_Char_PrintString("                ");
            LCD_Char_Position(1,0);
            LCD_Char_PrintString("                ");
    
            LCD_Char_Position(0,0);
            LCD_Char_PrintString("sw data         ");
            LCD_Char_Position(1,0);
            LCD_Char_PrintInt32(sw_data);
            CyDelay(100);
 */         
            if (no_hold) no_hold = hold_check();

            if (!no_hold) // volume, rate change mode
	        {
                no_hold = 1;
	            if (sw_in==PLAY_STOP)
                {
		            if (!on_play)
		                sw_data = volume_rate_change();
		            else
                    {
		                sw_data = play_volume_change();
                    }

                    READ_SWITCH;
                    while(sw_in)
                    {
                        READ_SWITCH;
                        CyDelay(500);
                    }
                }
	        }
            else
            {
                if (sw_data==PLAY_STOP)
                {
                    if (!on_play)
		                play_mode = ~play_mode;
	                CyDelay(500);
                }
            }

            if (play_mode)
                PLAY_LED_ON;  //repeat
            else
                PLAY_LED_OFF; //normal(loop)

            while (sw_data != PLAY_STOP)
            {
                if (sw_data != REC)
                {
                    if (!audio_mode)
                    {
                        audio_mode = 1;  //Play mode
                        TLV320_play();
                        FORCE_PLAY_MODE;
                    }

                    if(sw_data==PLAY_UP)
                    {
                        next_file_direct = 0;
                        next_file_no ++;
                    }
                    if(sw_data==PLAY_DOWN)
                    {
                        next_file_direct = 1;
                        if (!next_file_no)
                        {
                            next_file_no = 1;
                            next_file_direct = 0;
                        }
                        else
                            next_file_no --;

                        if (!next_file_no)
                            next_file_no = max_file_no;
                    }
                }

                on_play = 1;
                sw_data = audio_play();

                if (!audio_mode) //Rec mode
                {
	                max_file_no ++;
                    sw_data = PLAY_STOP;
	            }

                if ((audio_mode)&&(!sw_data)) //Play mode
                {
                    if (!play_mode)        //normal(loop) 
                        sw_data = PLAY_UP;
                    else                   //repeat
                    {
                        next_file_no ++;
                        sw_data = PLAY_DOWN;
                    }
                }

                no_hold = hold_check();
                
            } // end of while

            if (audio_mode)
            {
                audio_mode = 0;  //Rec mode
                TLV320_rec();
                FORCE_REC_MODE;
            }

        }
        else
            on_play = 0;
    }

    if (err)
    {
        while(1)
        {
           Control_Reg_1_Write(0x43); //all LED on
           CyDelay(250);
           Control_Reg_1_Write(0); //all LED off
           CyDelay(250);
        }
    }
  
    FORCE_CF_CS_HIGH;   
    FORCE_CF_RESET_LOW;
    
    return (0);
}

/* [] END OF FILE */
